@@ -235,45 +235,10 @@ g3l_distribution <- function (
         obsstock__wgt <- g3_stock_instance(obsstock, ld$weight)
     }
 
-    # If there are no fleets, do abundance comparison
-    if (length(fleets) == 0) for (stock in stocks) {
-        # Work out stock index for obs/model variables
-        if (!is.null(ld$stock_map)) {
-            # Skip over stocks not part of the observation data, map to an index
-            # NB: This is what stock_iterate() would do for us
-            if (is.null(ld$stock_map[[stock$name]])) next
-            stockidx_f <- f_substitute(~g3_idx(x), list(x = ld$stock_map[[stock$name]]))
-        } else {
-            # Not using stock grouping, __stock_idx variable not needed
-            stockidx_f <- ~-1
-        }
+    # If no fleets, set fleet_stock = NULL, otherwise iterate over fleets
+    for (fleet_stock in (if (length(fleets) > 0) fleets else list(NULL))) for (prey_stock in stocks) {
+        stock <- prey_stock  # Alias stock == prey_stock
 
-        # Collect all of stock and dump it in modelstock
-        out[[step_id(run_at, 'g3l_distribution', nll_name, 1, stock)]] <- f_substitute(~{
-            debug_label(prefix, "Collect abundance from ", stock, " for ", nll_name)
-            stock_iterate(stock, stock_intersect(modelstock, {
-                if (compare_num) {
-                    debug_trace("Add ", stock, " individuals to our count")
-                    stock_ss(modelstock__num) <- stock_ss(modelstock__num) +
-                        stock_reshape(modelstock, stock_ss(stock__num))
-                }
-                if (compare_wgt) {
-                    debug_trace("Take ", stock, " total biomass to our count")
-                    stock_ss(modelstock__wgt) <- stock_ss(modelstock__wgt) +
-                        stock_reshape(modelstock, stock_ss(stock__num) * stock_ss(stock__wgt))
-                }
-            }))
-        }, list(
-            compare_num = !is.null(ld$number),
-            compare_wgt = !is.null(ld$weight)))
-
-        # Fix-up stock intersection, add in stockidx_f
-        out[[step_id(run_at, 'g3l_distribution', nll_name, 1, stock)]] <- g3_step(f_substitute(out[[step_id(run_at, 'g3l_distribution', nll_name, 1, stock)]], list(
-            stockidx_f = stockidx_f)))
-    }
-
-    # Otherwise, do fleet catch comparison
-    for (fleet_stock in fleets) for (prey_stock in stocks) {
         # Work out stock index for obs/model variables
         if (!is.null(ld$stock_map)) {
             # Skip over stocks not part of the observation data, map to an index
@@ -296,26 +261,83 @@ g3l_distribution <- function (
             fleetidx_f <- ~-1
         }
 
-        # Collect all of fleet's sampling of prey and dump it in modelstock
-        out[[step_id(run_at, 'g3l_distribution', nll_name, 1, fleet_stock, prey_stock)]] <- g3_step(f_substitute(~{
-            debug_label(prefix, "Collect catch from ", fleet_stock, "/", prey_stock, " for ", nll_name)
-            stock_iterate(prey_stock, stock_intersect(modelstock, {
+        # Inner formula to do collection
+        if (is.null(fleet_stock)) {
+            # No fleet -> Compare to abundance
+            collect_f <- f_substitute(~{
+                if (compare_num) {
+                    debug_trace("Add ", prey_stock, " individuals to our count")
+                    stock_ss(modelstock__num) <- stock_ss(modelstock__num) + stock_reshape(modelstock,
+                        transform_f * tform_stock_ss(prey_stock__num))
+                }
+                if (compare_wgt) {
+                    debug_trace("Take ", prey_stock, " total biomass to our count")
+                    stock_ss(modelstock__wgt) <- stock_ss(modelstock__wgt) + stock_reshape(modelstock,
+                        transform_f * tform_stock_ss(prey_stock__num) * tform_stock_ss(prey_stock__wgt))
+                }
+            }, list(
+                compare_num = !is.null(ld$number),
+                compare_wgt = !is.null(ld$weight)))
+        } else {
+            collect_f <- f_substitute(~{
                 if (compare_num) {
                     debug_trace("Take prey_stock__predby_fleet_stock weight, convert to individuals, add to our count")
-                    stock_ss(modelstock__num) <- stock_ss(modelstock__num) +
-                        stock_reshape(modelstock, stock_ss(prey_stock__predby_fleet_stock) / avoid_zero_vec(stock_ss(prey_stock__wgt)))
+                    stock_ss(modelstock__num) <- stock_ss(modelstock__num) + stock_reshape(modelstock,
+                        transform_f * tform_stock_ss(prey_stock__predby_fleet_stock) / avoid_zero_vec(tform_stock_ss(prey_stock__wgt)))
                 }
                 if (compare_wgt) {
                     debug_trace("Take prey_stock__predby_fleet_stock weight, add to our count")
-                    stock_ss(modelstock__wgt) <- stock_ss(modelstock__wgt) +
-                        stock_reshape(modelstock, stock_ss(prey_stock__predby_fleet_stock))
+                    stock_ss(modelstock__wgt) <- stock_ss(modelstock__wgt) + stock_reshape(modelstock,
+                        transform_f * tform_stock_ss(prey_stock__predby_fleet_stock))
                 }
-            }))
-        }, list(
-            compare_num = !is.null(ld$number),
-            compare_wgt = !is.null(ld$weight),
-            # Find catch from predation step
-            prey_stock__predby_fleet_stock = as.symbol(paste0('prey_stock__predby_', fleet_stock$name)))), recursing = TRUE)
+            }, list(
+                compare_num = !is.null(ld$number),
+                compare_wgt = !is.null(ld$weight),
+                prey_stock__predby_fleet_stock = as.symbol(paste0('prey_stock__predby_', fleet_stock$name))))
+        }
+
+        # Wrap with any iteration for transformations
+        transform_f <- quote(1)
+        tform_ss <- list()
+        for (tf_index in seq_along(transform_fs)) {
+            tf_name <- names(transform_fs)[[tf_index]]
+            transform_f <- f_substitute(quote(transform_f * extra_tf), list(
+                transform_f = transform_f,
+                extra_tf = transform_fs[[tf_index]]))
+
+            if (tf_name == 'age') {
+                # NB: This is nearly interact$age, but not quite. We need a different _idx name
+                collect_f <- f_substitute(quote({
+                    # NB: By setting the iterator to preage, we may remove mentions of __age_idx,
+                    #     which will cause stock_iterate() to not loop over age. Force this by
+                    #     explicitly mentioning the index.
+                    comment(prey_stock__age_idx)
+                    for (preage in seq(prey_stock__minage, prey_stock__maxage, by = 1)) g3_with(
+                        prey_stock__preage_idx := g3_idx(preage - prey_stock__minage + 1L),
+                        collect_f)
+                }), list(collect_f = collect_f))
+                # Add extra subset for injection below
+                tform_ss$age <- quote( stock__preage_idx )
+            } else {
+                stop("Transforms for dimensions other than age not supported yet")
+            }
+        }
+
+        # Replace tform_stock_ss, adding in the transform subsets required
+        collect_f <- call_replace(collect_f, tform_stock_ss = function (x) {
+            as.call(c(
+                list(as.symbol("stock_ss"), x[[2]]),
+                tform_ss))
+        })
+        # Add in transform_f now we're done adding bits to it.
+        collect_f <- f_substitute(collect_f, list(transform_f = transform_f))
+
+        # Finally iterate/intersect over stock in question
+        out[[step_id(run_at, 'g3l_distribution', nll_name, 1, fleet_stock, prey_stock)]] <- f_optimize(f_substitute(~{
+            if (compare_fleet) debug_label(prefix, "Collect catch from ", fleet_stock, "/", prey_stock, " for ", nll_name)
+            if (!compare_fleet) debug_label(prefix, "Collect abundance from ", stock, " for ", nll_name)
+            stock_iterate(prey_stock, stock_intersect(modelstock, collect_f))
+        }, list(compare_fleet = !is.null(fleet_stock), collect_f = collect_f)))
 
         # Fix-up stock intersection, add in stockidx_f
         out[[step_id(run_at, 'g3l_distribution', nll_name, 1, fleet_stock, prey_stock)]] <- f_substitute(out[[step_id(run_at, 'g3l_distribution', nll_name, 1, fleet_stock, prey_stock)]], list(
@@ -324,54 +346,10 @@ g3l_distribution <- function (
         out[[step_id(run_at, 'g3l_distribution', nll_name, 1, fleet_stock, prey_stock)]] <- g3_step(out[[step_id(run_at, 'g3l_distribution', nll_name, 1, fleet_stock, prey_stock)]])
     }
 
-    # Add any required transforms
-    for (tf_index in seq_along(transform_fs)) {
-        tf_name <- names(transform_fs)[[tf_index]]
-        if (tf_name == 'age') {
-            # NB: This is nearly interact$age, but not quite. We need a different _idx name
-            transform_f <- f_substitute(quote(
-                for (preage in seq(modelstock__minage, modelstock__maxage, by = 1)) g3_with(
-                    modelstock__preage_idx := g3_idx(preage - modelstock__minage + 1L), {
-                        stock_ss(modelstock__x) <- stock_ss(modelstock__x) + transform_f *
-                            stock_ss(modelstock__x_pretransform, age = modelstock__preage_idx)
-                    })
-            ), list(transform_f = transform_fs[[tf_index]]))
-        } else {
-            stop("Transforms for dimensions other than age not supported yet")
-        }
-
-        # Wrap again with common zeroing code
-        transform_f <- g3_step(f_substitute(~{
-            debug_label(prefix, "Transform ", modelstock, " ", tf_name)
-            if (done_aggregating_f) {
-                stock_iterate(modelstock, {
-                    stock_ssinv(modelstock__x_pretransform, 'time') <- stock_ssinv(modelstock__x, 'time')
-                    stock_ssinv(modelstock__x, 'time') <- 0
-                })
-                stock_iterate(modelstock, transform_f)
-            }
-        }, list(
-            done_aggregating_f = ld$done_aggregating_f,
-            transform_f = transform_f,
-            tf_name = tf_name)))
-
-        if (!is.null(ld$number)) {
-            modelstock__num_pretransform <- g3_stock_instance(modelstock, 0)
-            out[[step_id(run_at, 'g3l_distribution', nll_name, 2, tf_index, 'num')]] <-
-                generic_var_replace(transform_f, 'num')
-        }
-
-        if (!is.null(ld$weight)) {
-            modelstock__wgt_pretransform <- g3_stock_instance(modelstock, 0)
-            out[[step_id(run_at, 'g3l_distribution', nll_name, 2, tf_index, 'wgt')]] <-
-                generic_var_replace(transform_f, 'wgt')
-        }
-    }
-
     nllstock <- g3_storage(paste("nll", nll_name, sep = "_"))
     if (nll_breakdown) nllstock <- g3s_modeltime(nllstock)
-    nllstock__num <- g3_stock_instance(nllstock, 0)
-    nllstock__wgt <- g3_stock_instance(nllstock, 0)
+    if (!is.null(ld$number)) nllstock__num <- g3_stock_instance(nllstock, 0)
+    if (!is.null(ld$weight)) nllstock__wgt <- g3_stock_instance(nllstock, 0)
     nllstock__weight <- g3_stock_instance(nllstock, 0)
     nll <- 0.0
 
